#!/home/lukas/dev/COVID-19/csse_covid_19_data/csse_covid_19_daily_reports/venv/bin/python3
import requests
import collections
import datetime
import pytz
import pprint
import csv
import os
assert pprint

stations = collections.OrderedDict()
# our name = (wu station name, timezone)
stations["Prague_LKPR"] = ("LKPR:9:CZ", "Europe/Prague")  # Prague Aiport; Czech Republic

stations["Canada_Alberta"] = ("CYEG:9:CA", "MST")
stations["Canada_British_Columbia"] = ("CYXS:9:CA", "Canada/Pacific")
stations["Canada_Nova_Scotia"] = ("CYHZ:9:CA", "Canada/Atlantic")  # IGRANDLA84
stations["Canada_Ontario"] = ("KONT:9:US", "EST")  # KCAONTAR21
stations["Canada_Quebec"] = ("CYQB:9:CA", "EST")  # IQUEBE57

stations["Serbia"] = ("LYBE:9:RS", "Europe/Belgrade")  # Belgrade
stations["Ukraine"] = ("UKKK:9:UA", "Europe/Kiev")  # kiyv
stations["Switzerland"] = ("LSGG:9:CH", "Europe/Berlin")  # geneva
stations["Moldova"] = ("LUKK:9:MD", "Europe/Chisinau")  # LUKK

stations["Iceland"] = ("BIRK:9:IS", "Atlantic/Reykjavik")
stations["Singapore"] = ("WSSS:9:SG", "Asia/Singapore")
stations["Taiwan"] = ("RCSS:9:TW", "Asia/Taipei")
stations["New_Zealand"] = ("NZWN:9:NZ", "NZ")  # wellington IWELLING323

stations["South_Korea"] = ("RKSS:9:KR", "Asia/Seoul")  # seoul
stations["Vietnam"] = ("VVTS:9:VN", "Asia/Ho_Chi_Minh")  # ho-chi-minh-city
stations["Seychelles"] = ("FSIA:9:SC", "Indian/Mauritius")  # FSIA
stations["South Africa"] = ("FAGM:9:ZA", "Africa/Johannesburg")  # johannesburg
stations["Uruguay"] = ("SUMU:9:UY", "America/Montevideo")  # montevideo
stations["Kazakhstan"] = ("UACC:9:KZ", "Asia/Almaty")  # astana

stations["Saudi Arabia"] = ("OERY:9:SA", "Asia/Riyadh") # OERY
# stations["Israel"] = ("LLSD:9:IL", "Asia/Tel_Aviv")  # tel nof

stations["Netherlands"] = ("EHAM:9:NL", "Europe/Berlin")  # amsterdam
stations["Italy"] = ("LIML:9:IT", "Europe/Berlin")  # milan
stations["Ireland"] = ("EIDW:9:IE", "Europe/Dublin") # dublin
stations["Denmark"] = ("EKCH:9:DK", "Europe/Berlin")  # copenhagen
stations["Spain"] = ("LEMD:9:ES", "Europe/Berlin")  # madrid
stations["Luxembourg"] = ("ELLX:9:LU", "Europe/Berlin")  # luxembourg
stations["Slovenia"] = ("LJLJ:9:SI", "Europe/Berlin")   # ljubljana
stations["Greece"] = ("LGAV:9:GR", "Europe/Berlin")  # athens
stations["Sweden"] = ("ESSB:9:SE", "Europe/Berlin")  # stockholm
stations["Malta"] = ("LMML:9:MT", "Europe/Malta")  # velletta

stations["Alabama"] = ("KMGM:9:US", "US/Central")  # Montgomerry KALMONTG119
stations["Alaska"] = ("PAJN:9:US", "US/Alaska")  # KAKJUNEA63
stations["California"] = ("KBUR:9:US", "US/Pacific")  # LA  KCALOSAN880
stations["Colorado"] = ("KDEN:9:US", "US/Mountain") # KCODENVE765
stations["New York"] = ("KLGA:9:US", "US/Eastern")  # la guardia
stations["Texas"] = ("KAUS:9:US", "US/Central")  # Austin KTXAUSTI1619
stations["Virginia"] = ("KRIC:9:US", "US/Eastern")  # Richmond  KVARICHM154
stations["Florida"] = ("KTLH:9:US", "US/Eastern")  # KFLTALLA166
stations["Washington"] = ("KSEA:9:US", "US/Pacific")  # Olympia KWAOLYMP300

s = None

def init_shit():
    global s 
    s = requests.session()
    s.headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36',
        'referer': "https://www.wunderground.com/history/monthly/cz/prague/LKPR/date/2020-2",
    }
    s.get("https://www.wunderground.com/history/monthly/cz/prague/LKPR/date/2020-2")

def get_date(time_, timezone_):
    # eg. 1580509800
    # -> parse date as MMDD, use timezone_ when converting
    dt = datetime.datetime.fromtimestamp(time_, timezone_)
    return dt.strftime("%Y%m%d")

def fetch_station(station, timezone_, start_date, end_date):
    if s is None:
        init_shit()
#    r = s.get("https://api.weather.com/v1/location/LKPR:9:CZ/almanac/daily.json?apiKey=6532d6454b8aa370768e63d6ba5a832e&units=m&start=0101&end=0630")
    r = s.get(f"https://api.weather.com/v1/location/{station}/observations/historical.json?apiKey=6532d6454b8aa370768e63d6ba5a832e&units=m&startDate={start_date}&endDate={end_date}")
    temps = collections.OrderedDict()
    if 'errors' in r.json():
        print(r.json()['errors'])
        print("skipping.")
        return {}
    for data in r.json()["observations"]:
        # use timezone to get date
        time_ = data['valid_time_gmt']  # eg. 1580509800
        date_ = get_date(time_, timezone_)
        if date_ in temps:
            temps[date_].append(data['temp'])
        else:
            temps[date_] = [data['temp']]

    for date_, ts in temps.items():
        tss_ = [x for x in temps[date_] if x is not None]
        if not tss_:
            temps[date_] = None
        else:
            temps[date_] = "%.2f" % (sum(tss_) / float(len(tss_)),)
    return temps
    #d = collections.OrderedDict()
    #for data in r.json()['almanac_summaries']:
    #    d[data['almanac_dt']] = data['mean_temp']
    #return d

def get_needed_dates(last_day_fetched, today):
    from_ = datetime.datetime.strptime(last_day_fetched, "%Y%m%d")
    to_ = datetime.datetime.strptime(today, "%Y%m%d")
    current_from = from_
    while True:
        current_to = current_from + datetime.timedelta(days=29)
        if current_to >= to_:
           current_to = to_
        yield (current_from.strftime("%Y%m%d"), current_to.strftime("%Y%m%d"))
        if current_to == to_:
            break
        current_from = current_to + datetime.timedelta(days=1)

today = datetime.datetime.now().strftime("%Y%m%d")
for station, (wu_name, timezone_) in stations.items():
    timezone_ = pytz.timezone(timezone_)

    last_day_fetched = "20191231"
    if os.path.exists(f"temps/{station}.csv"):
        with open(f"temps/{station}.csv", "r") as f:
            c = csv.reader(f)
            for line in c:
                last_day_fetched = line[0]

    if (datetime.datetime.strptime(today, "%Y%m%d") - 
            datetime.datetime.strptime(last_day_fetched,"%Y%m%d")).days <= 1:
        print(f"All data fetched for {station} (last_day_fetched: {last_day_fetched})")
        continue

    days_str = ', '.join(x+":"+y for x,y in get_needed_dates(last_day_fetched, today))
    print(f"Fetching {station} for days {days_str}")

    data = collections.OrderedDict()
    for sd, ed in get_needed_dates(last_day_fetched, today):
        data_ = fetch_station(wu_name, timezone_, sd, ed)
        data.update(data_)

    if not os.path.exists(f"temps/{station}.csv"):
        with open(f"temps/{station}.csv", "w") as f:
            csvwriter = csv.writer(f)
            csvwriter.writerow(["date", "mean_temp"])
            for d,t in data.items():
                csvwriter.writerow([d, t])
    else:
        with open(f"temps/{station}.csv", "a") as f:
            csvwriter = csv.writer(f)
            for d,t in data.items():
                csvwriter.writerow([d, t])
